class Pettanr.Views.Sheet.PlayModule
class Pettanr.Views.Sheet.PlayModule.Footer extends Pettanr.View
  tagName: 'table'
  className: 'no-border'
  
  initialize: (options) ->
    @panel = options.panel
    @sheet_panel = options.sheet_panel
  
  render: () ->
    this.$el.html('')
    this.$el.append(
      (new Pettanr.Views.Panel.Icon({item: @panel, half: true})).render().el
    )
    this.$el.append(
      (new Pettanr.Views.Author.Icon({item: @panel.author(), half: true})).render().el
    )
    this.$el.append(Pettanr.AppHelper.distance_of_time_in_words_to_now(@panel.get('updated_at')))
    #
    this.$el.append(
      (new Pettanr.Views.Panel.Icon({item: @sheet_panel, half: true})).render().el
    )
    this.$el.append(
      (new Pettanr.Views.Author.Icon({item: @panel.author(), half: true})).render().el
    )
    this.$el.append(Pettanr.AppHelper.distance_of_time_in_words_to_now(@sheet_panel.get('updated_at')))
    #
    if @panel.is_own()
      this.$el.append('')
    this
  
class Pettanr.Views.Sheet.PlayModule.Panel extends Pettanr.View
  tagName: 'div'
  
  initialize: (options) ->
    @sheet_panel = options.sheet_panel
  
  clear: () ->
    this.$el.html('')
    @sheet_panel.get_parent('panel', this, {
      success: (@panel) =>
        @render()
    })
    this
  
  render: () ->
    this.$el.html('')
    body = new Pettanr.Views.Panel.Body({
      panel: @panel,
      spot: null
    })
    this.$el.append(body.render().el)
    footer = new Pettanr.Views.Panel.Footer({
      sheet_panel: sheet_panel,
      panel: @panel,
    })
    this.$el.append(footer.render().el)
    this.credits.push(@panel.licensed_pictures())
    this
  
class Pettanr.Views.Sheet.PlayModule.Panels extends Pettanr.View
  tagName: 'div'
  
  initialize: (options) ->
    @parent = options.parent
    @items = options.items
    @credits = options.credits
  
  render: () ->
    this.$el.html('')
    _.each @items, (sheet_panel) =>
      return if not sheet_panel.has_panel()
      panel_view = new Pettanr.Views.Sheet.PlayModule.Panel({
        sheet_panel: sheet_panel,
      })
      this.$el.append(panel_view.render().el)
    this
  
class Pettanr.Views.Sheet.PlayModule.Body extends Pettanr.View
  tagName: 'div'
  
  initialize: (options) ->
    @parent = options.parent
    @list = options.list
    @items = @list.items()
    if @items.length < 1
      @render_empty()
    else
      @render()
  
  render_empty: () ->
    this.$el.html('')
    this.$el.append((new Tag.H2({content: I18n.t('sheets.play.empty')})).render().el)
    this
  
  render: () ->
    this.$el.html('')
    # paginate(@pager)
    credits = new Pettanr.View.Credits({parent: this})
    panels = new Pettanr.Views.Sheet.PlayModule.Panels({
      parent: this,
      items: @items,
      credits: credits,
    })
    this.$el.append(panels.render().el)
    this.$el.append(credits.render().el)
    @append_rb()
    # paginate(@pager)
    this
  
class Pettanr.Views.Sheet.Play extends Pettanr.View.Show
  
  initialize: (@item, options) ->
    @list = options.list
    @header = new Pettanr.View.Show.Header(@item, this, @binder_header_options())
    @authored_by = @item.authored_by()
    @body = new Pettanr.Views.Sheet.PlayModule.Body({
      parent: this,
      list: @list, 
    })
    @owner = new Pettanr.Views.Sheet.PlayModule.Owner({item: @item})
    @listenTo(@authored_by, 'click', @click_authored_by)
  
  render: () ->
    this.$el.html('')
    this.$el.append(@header.render().el)
    this.$el.append(@authored_by.render().el)
    this.$el.append(@body.render().el)
    this.$el.append(@owner.render().el) if @item.is_own()
    this
  
