#サポートツール
require 'rubygems'
require 'active_support/all'
require 'rest_client'
require 'yaml'
require 'json'
require 'base64'

if ARGV.size < 1
  puts "panel_import.rb src_host src_email src_token dest_host dest_email dest_token"
  exit
end

src_host = ARGV[0]
src_email= ARGV[1]
src_token = ARGV[2]
dest_host = ARGV[3]
dest_email = ARGV[4]
dest_token = ARGV[5]

class Importer
  def initialize src_host, src_email, src_token, dest_host, dest_email, dest_token
    @src_host = src_host
    @src_email = src_email
    @src_token = src_token
    @dest_host = dest_host
    @dest_email = dest_email
    @dest_token = dest_token
    @src_licenses = {}
    self.licenses(@src_host).each do |l|
      @src_licenses[l['id'].to_i] = l
    end
    @dest_licenses = {}
    self.licenses(@dest_host).each do |l|
      @dest_licenses[l['name']] = l
    end
    @src_speech_balloon_templates = {}
    self.speech_balloon_templates(@src_host, @src_email, @src_token).each do |sb|
      @src_speech_balloon_templates[sb['id'].to_i] = sb
    end
    @dest_speech_balloon_templates = {}
    self.speech_balloon_templates(@dest_host, @dest_email, @dest_token).each do |sb|
      @dest_speech_balloon_templates[sb['name']] = sb
    end
    @src_writing_formats = {}
    self.writing_formats(@src_host, @src_email, @src_token).each do |wf|
      @src_writing_formats[wf['id'].to_i] = wf
    end
    @dest_writing_formats = {}
    self.writing_formats(@dest_host, @dest_email, @dest_token).each do |wf|
      @dest_writing_formats[wf['name']] = wf
    end
    @src_system_pictures = {}
    self.system_pictures(@src_host, @dest_email, @dest_token).each do |sp|
      @src_system_pictures[sp['id'].to_i] = sp
    end
    @dest_system_pictures = {}
    self.system_pictures(@dest_host, @dest_email, @dest_token).each do |sp|
      @dest_system_pictures[sp['md5']] = sp
    end
  end
  
  def licenses host
    res = {}
    u = host + 'licenses.json'
    r = RestClient.get(u, :params => {:page_size  => 100})
    j = JSON.parse r
    j['list']
  end
  
  def license_groups host
    res = {}
    u = host + 'license_groups.json'
    r = RestClient.get(u, :params => {:page_size  => 100})
    j = JSON.parse r
    ary = j['list']
    ary.each do |l|
      res[l['id'].to_i] = l
    end
    res
  end
  
  def speech_balloon_templates host, email, token
    res = {}
    u = host + 'speech_balloon_templates.json'
    r = RestClient.get(u, {:params => {:page_size  => 100}, :email => email, :token => token})
    j = JSON.parse r
    j['list']
  end
  
  def writing_formats host, email, token
    res = {}
    u = host + 'writing_formats.json'
    r = RestClient.get(u, {:params => {:page_size  => 100}, :email => email, :token => token})
    j = JSON.parse r
    j['list']
  end
  
  def system_pictures host, email, token
    res = {}
    u = host + 'system_pictures.json'
    r = RestClient.get(u, {:params => {:page_size  => 100}, :email => email, :token => token})
    j = JSON.parse r
    j['list']
  end
  
  def panels host, email, token
    res = {}
    u = host + 'home/panels.json'
    r = RestClient.get(u, {:params => {:page_size  => 100}, :email => email, :token => token})
    resp = JSON.parse r
    status = resp['page_status']
    resp['list'].each do |panel|
      res[panel['id'].to_i] = panel
    end
    res
  end
  
  def panel pid, host, email, token
    res = {}
    u = host + 'panels/' +  pid.to_s + '.json'
    r = RestClient.get(u, {:params => {:with_elements  => true}, :email => email, :token => token})
    j = JSON.parse r
  end
  
  def picture id, host, email, token
    u = host + 'pictures/' + id.to_s + '.json'
    r = RestClient.get(u, {:email => email, :token => token})
    JSON.parse r
  end
  
  def search_resource_picture md5, host, email, token
    res = {}
    u = host + 'resource_pictures/search.json'
    r = RestClient.get(u, {:params => {:md5 => md5}, :email => email, :token => token})
    JSON.parse r
  end
  
  def prepare_panel_pictures panel_picture
    element_name = 'panel_pictures'
    puts 'panel_picture:' + panel_picture['id'].to_s
    # get picture from src
    picture = self.picture panel_picture['picture_id'], @src_host, @src_email, @src_token
    puts 'picture:' + picture['id'].to_s
    # get resource_picture if exists?
    # choose latest resource_picture if find same rp at server
    rps = self.search_resource_picture picture['md5'], @dest_host, @dest_email, @dest_token
    if rps.any?
      rp = rps.first
      puts 'resource picture ok:' + rp['id'].to_s + '(' + rp['md5'].to_s + ')'
    else
      puts 'no picture'
      c = false
      return
    end
    # get lisence's id if exists?
    src_ls_name = @src_licenses[picture['license_id']]['name']
    puts 'license:' + picture['license_id'].to_s + '(' + src_ls_name + ')'
    if dl = @dest_licenses[src_ls_name]
      puts 'license ok(' + dl['id'].to_s + ')'
    else
      puts 'no license'
      c = false
      return
    end
    panel_picture['picture_id'] = rp['picture_id']
    true
  end
  
  def prepare_speech_balloons speech_balloon
    element_name = 'speech_balloons'
    puts 'sb:' + speech_balloon['id'].to_s
    src_sb_name = @src_speech_balloon_templates[speech_balloon['speech_balloon_template_id']]['name']
    puts 'speech_balloon_template:' + speech_balloon['speech_balloon_template_id'].to_s + '(' + src_sb_name + ')'
    if dsb = @dest_speech_balloon_templates[src_sb_name]
      puts 'speech_balloon_template ok(' + dsb['id'].to_s + ')'
    else
      puts 'no speech_balloon_template'
      c = false
      return
    end
    src_sp_md5 = @src_system_pictures[speech_balloon['balloon']['system_picture_id']]['md5']
    puts 'balloon picture:' + speech_balloon['balloon']['system_picture_id'].to_s + '(' + src_sp_md5 + ')'
    if dsp = @dest_system_pictures[src_sp_md5]
      puts 'balloon picture ok(' + dsp['id'].to_s + ')'
    else
      puts 'no balloon picture'
      c = false
      return
    end
    src_wf_name = @src_writing_formats[speech_balloon['speech']['writing_format_id']]['name']
    puts 'speech writing_format:' + speech_balloon['speech']['writing_format_id'].to_s + '(' + src_wf_name + ')'
    if dwf = @dest_writing_formats[src_wf_name]
      puts 'speech writing_format ok(' + dwf['id'].to_s + ')'
    else
      puts 'no speech writing_format'
      c = false
      return
    end
    speech_balloon['speech_balloon_template_id'] = dsb['id']
    speech_balloon["speech_balloon_template_settings"] = speech_balloon["settings"]
    speech_balloon.delete 'settings'
    speech_balloon["speech_balloon_template_module_name"] = dsb["module_name"]
    speech_balloon.delete 'module_name'
    #e.delete 'speech_balloon_template'
    balloon = speech_balloon['balloon']
    speech_balloon.delete 'balloon'
    balloon.delete 'id'
    balloon.delete 'speech_balloon_id'
    balloon["speech_balloon_template_settings"] = speech_balloon["settings"]
    balloon.delete 'settings'
    balloon['system_picture_id'] = dsp['id']
    balloon['speech_balloon_template_id'] = dsb['id']
    balloon["speech_balloon_template_module_name"] = dsb["module_name"]
    
    speech = speech_balloon['speech']
    speech_balloon.delete 'speech'
    speech.delete 'id'
    speech.delete 'speech_balloon_id'
    speech["speech_balloon_template_settings"] = speech_balloon["settings"]
    speech.delete 'settings'
    speech['speech_balloon_template_id'] = dsb['id']
    speech["speech_balloon_template_module_name"] = dsb["module_name"]
    speech['writing_format_id'] = dwf['id']
    speech["writing_format_module_name"] = dwf["module_name"]
    
    speech_balloon['balloon_attributes'] = balloon
    speech_balloon['speech_attributes'] = speech
    true
  end
  
  def prepare_ground_colors ground_color
    element_name = 'ground_colors'
    puts 'ground_color:' + ground_color['id'].to_s
    true
  end
  
  def prepare_ground_pictures ground_picture
    element_name = 'ground_pictures'
    puts 'ground_picture:' + ground_picture['id'].to_s
    rps = self.resource_picture ground_picture['picture']['md5'], @dest_host, @dest_email, @dest_token
    if rps.any?
      rp = rps.first
      puts 'picture ok(' + rp['id'].to_s + ')'
    else
      puts 'no picture'
      c = false
      return
    end
    #e['license_id'] = dl['id']
    ground_picture['picture_id'] = rp['id']
    true
  end
  
  def add_attribute element_name, panel_json, attribute
    eid = attribute['id']
    attribute.delete 'id'
    attribute.delete 'panel_id'
    panel_json[element_name + '_attributes'] ||= []
    panel_json[element_name + '_attributes'].push attribute
  end
  
  def edit_elements panel_json, element_name
    r = true
    (panel_json[element_name] || {}).each do |attr|
      r = self.__send__ 'prepare_' + element_name, attr
      break unless r
      self.add_attribute element_name, panel_json, attr
    end
    panel_json.delete element_name
    r
  end
  
  def each_panel
    self.panels(@src_host, @src_email, @src_token).each do |key, attrs|
      puts 'source panel:' + key.to_s
      panel_json = self.panel(key, @src_host, @src_email, @src_token)
      r = true
      ['panel_pictures', 'speech_balloons', 'ground_pictures', 'ground_colors'].each do |element_name|
        r = self.edit_elements(panel_json, element_name)
        break unless r
      end
      next unless r
      puts '============ importable'
      panel_json.delete 'id'
      panel_json.delete 'author'
      panel_json.delete 'author_id'
      panel_json.delete "created_at"
      panel_json.delete 'updated_at'
      yield panel_json
    end
  end
  
end

u = Importer.new src_host, src_email, src_token, dest_host, dest_email, dest_token

u.each_panel do |panel_json|
  u = dest_host + 'panels'
  # write log
  File.open('/Sites/panel_create.json', 'w') do |f|
   f.write JSON.pretty_generate(panel_json)
  end
  begin
    r = RestClient.post(u, 
      {:panel => panel_json}.to_json, 
      :content_type => :json, :accept => :json, :email => dest_email, :token => dest_token
    )
    res = JSON.parse r
  rescue
    puts ' failed: ' + r.to_s
  end
end


